#!/usr/bin/perl -I. -w
use strict;

#  ---------------------------------------------------------------------
#   configuration script for DBReview v1.0 
#    v1.0
#    November 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------

# configure the OAI data provider layer


use lib '../lib/';

use Pure::EZXML;
use Pure::EZHTTP;
use Pure::X2D;


# get data through HTTP, with retries
sub getHTTP
{
   my ($url) = @_;

   # create a user agent object
   my $ua = new Pure::EZHTTP;
   $ua->agent("VT OAI Harvester/1.0 " . $ua->agent);

   # create a request
   my $req = new Pure::EZHTTP GET => $url;

   my $state = 0;
   my $res;
   while ($state == 0)
   {
      # pass request to the user agent and get a response back
      $res = $ua->request($req);
      
      if ($res->code == 503)
      {
         my $sleep = $res->header ('Retry-After');
         if (not defined ($sleep) || ($sleep < 0) || ($sleep > 86400))
         { $state = 1;}
         else
         { sleep ($sleep); }
      }
      else
      { $state = 1; }
   }
   
   # return results to caller
   $res;
}


# get a single configuration option from the parsed XML file
sub getOption
{
   my ($doc, $option, $embed) = @_;
   
   if (defined $embed)
   {
      my @embedparts = split ('/', $embed);
      for ( my $i=0; $i<=$#embedparts; $i++ )
      {
         my $element = $doc->getElementsByTagName ($embedparts[$i], 0);
         if ($element->getLength > 0)
         {
            $doc = $element->item(0);
         }
         else
         {
            return '';
         }
      }
   }
   
   my $element = $doc->getElementsByTagName ($option, 0);
   if ($element->getLength > 0)
   {
      my $instance = $element->item(0);
      return $instance->getChildNodes->toString;
   }
   else
   {
      return '';
   }
}


# read in configuration from config.xml and set best-guess defaults for 
# missing values
sub ReadConfig
{
   my ($configname) = @_;
   
   my $configfile = "$configname/config.xml";
   my $config = new Pure::X2D ($configfile);
   
   if ((! defined $config->{'repositoryName'}) || ($config->{'repositoryName'}->[0] eq ''))
   {
      $config->{repositoryName} = [ 'ODL Review Archive' ];
   }

   if ((! defined $config->{'adminEmail'}) || ($config->{'adminEmail'}->[0] eq ''))
   {
      $config->{'adminEmail'} = [ '' ];

      # guess admin id from hostname and user logged in
      my $hostname = `hostname`; 
      my $username = `whoami`;
      if ((defined $hostname) && ($hostname ne '') && 
          (defined $username) && ($username ne ''))
      {
         $hostname =~ s/[\r\n\t\s]//go;
         $username =~ s/[\n\r\t\s]//go;
         
         $config->{'adminEmail'} = [ $username.'@'.$hostname ];
      }
   }

   if ((! defined $config->{'database'}) || ($config->{'database'}->[0] eq ''))
   { $config->{'database'} = [ 'review' ]; }
   if ((! defined $config->{'dbusername'}) || ($config->{'dbusername'}->[0] eq ''))
   {
      $config->{'dbusername'} = [ '' ];
      
      my $username = `whoami`;
      if ((defined $username) && ($username ne ''))
      {
         $username =~ s/[\r\t\n\s]//go;
         $config->{'dbusername'} = [ $username ]; 
      }
   }
   if ((! defined $config->{'dbpassword'}) || ($config->{'dbpassword'}->[0] eq ''))
   { $config->{'dbpassword'} = [ '' ]; }
   if ((! defined $config->{'table'}) || ($config->{'table'}->[0] eq ''))
   { $config->{'table'} = [ $configname ]; }
   if ((! defined $config->{'recordlimit'}) || ($config->{'recordlimit'}->[0] eq ''))
   { $config->{'recordlimit'} = [ '1000' ]; }
   if ((! defined $config->{'archiveId'}) || ($config->{'archiveId'}->[0] eq ''))
   { $config->{'archiveId'} = [ $configname ]; }
   
   if ((! defined $config->{'allowread'}) || ($config->{'allowread'}->[0] eq ''))
   {
      $config->{'allowread'} = [ '' ];

      # guess machine name from hostname
      my $hostname = `hostname`; 
      if ((defined $hostname) && ($hostname ne ''))
      {
         $hostname =~ s/[\r\n\t\s]//go;
         $config->{'allowread'} = [ $hostname ];
      }
   }

   if ((! defined $config->{'allowwrite'}) || ($config->{'allowwrite'}->[0] eq ''))
   {
      $config->{'allowwrite'} = [ '' ];

      # guess machine name from hostname
      my $hostname = `hostname`; 
      if ((defined $hostname) && ($hostname ne ''))
      {
         $hostname =~ s/[\r\n\t\s]//go;
         $config->{'allowwrite'} = [ $hostname ];
      }
   }


   $config->{'mdorder'} = [ qw ( repositoryName adminEmail database dbusername dbpassword table recordlimit archiveId allowread allowwrite) ];

   $config;
}


# write configuration data and template files
sub WriteConfig 
{
   my ($config, $configname) = @_;
   
   mkdir ($configname);
   
   open (CONFIG, ">$configname/config.xml");
   print CONFIG "<?xml version=\"1.0\" ?>\n\n";
   print CONFIG "<odlsubmit xmlns=\"http://oai.dlib.vt.edu/ODL/Box/config\"\n".
                "    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n".
                "    xsi:schemaLocation=\"http://oai.dlib.vt.edu/ODL/Box/config\n".
                "                         http://oai.dlib.vt.edu/ODL/Box/config.xsd\"\n".
                ">\n\n";
   print CONFIG $config->toXML;
   print CONFIG "</odlsubmit>\n";
   close (CONFIG);
   
   system ("cp template/* $configname/");
}


my $fastforward = 0;

# input a line of data, with prompt and default value
sub InputLine
{
   my ($variable, $prompt) = @_;
   
   if (defined $prompt)
   {
      print "$prompt [$$variable] : ";
   }
   if ($fastforward == 1)
   {
      print "$$variable\n";
      select(undef, undef, undef, 0.25);
   }
   else
   {
      my $line = <STDIN>;
      chomp $line;
      if ($line eq '&continue')
      {
         $fastforward = 1;
         print "$$variable\n";
      }
      elsif ($line eq '&delete')
      {
         $$variable = '';
         InputLine ($variable, $prompt);
      }
      elsif ($line ne '')
      {
         $$variable = $line;
      }
   }
}


# wait for user to press ENTER
sub InputEnter
{
   my $temp = '';
   InputLine (\$temp);
}


# check validity of input and print error if necessary
sub InvalidInput
{
   my ($check, $errormsg) = @_;
   
   if ($check)
   {
      print "$errormsg\n";
      $fastforward = 0;
      return 1;
   }
   0;
}


# check for database operations on a single table
sub CheckTable
{
   my ($dbh, $table) = @_;

   my $sth = $dbh->prepare ("select count(*) from $table");
   if (! defined $sth)
   {
      return "Error preparing a select for table $table";
   }
   if (!($sth->execute))
   {
      $sth->finish;
      return "Error executing a select on table $table";
   }
   if (!($sth->fetchrow_hashref))
   {
      $sth->finish;
      return "Error getting row from table $table";
   }
   $sth->finish;
   return '';
}


# main program body !
sub main 
{
   $| = 1;
   my $ok;
   
   print "+-----------------------------------------+\n".
         "| Review Engine Configurator              |\n".
         "+-----------------------------------------+\n".
         "| Version 1.0 :: November 2002            |\n".
         "| Hussein Suleman <hussein\@vt.edu>        |\n".
         "| Digital Library Research Laboratory     |\n".
         "| www.dlib.vt.edu :: Virginia Tech        |\n".
         "------------------------------------------+\n\n";
         
   if (! exists $ARGV[0])
   {
      print "Missing configuration name\n\n";
      print "Syntax: configure.pl <configname>\n\n";
      exit;
   }
   
   my $config = ReadConfig ($ARGV[0]);
   
   print "Defaults/previous values are in brackets - press <enter> to accept those\n".
         "enter \"&delete\" to erase a default value\n".
         "enter \"&continue\" to skip further questions and use all defaults\n".
         "press <ctrl>-c to escape at any time (new values will be lost)\n".
         "\nPress <enter> to continue\n\n";
         
   InputEnter;

         
   # test for not finding DBI
   my $gotdbi = eval "require DBI";
   print "\n[DBI]\n";
   print "You need the DBI Perl module installed in order to use this script\n".
         "Checking ... ";
   if (! defined $gotdbi)
   {
      print "not installed !\n".
            "Please press CTRL-C and then install that module.\n".
            "You also need a module specific to your database e.g., DBD::mysql.\n".
            " \"man CPAN\" for help on how to do this.\n\n";
      exit;
   }
   else
   {
      print "ok\n\n";
   }
   

   # test for available driver
   my @dbi_drivers = DBI->available_drivers;
   my $driver_list = join (', ', @dbi_drivers);
   print "\n[DATABASE DRIVER]\n";
   print "DBI requires special drivers to access different databases. Please\n".
         "confirm that the list below contains the name of your database.\n".
         "If it is not here or you are not sure, press CTRL-C and install it\n";
   print "\n".$driver_list."\n\n";
   print "Press ENTER to continue\n";
   InputEnter;

   
   # Database Name + Username + Password
   $ok = 0;
   print "\n[DATABASE CONNECTION]\n";
   print "The ODLBox scripts need to connect to a database to store\n".
         "and retrieve data. For this you need to supply the name of the\n".
         "database in the format required by your database\n".
         "(you will be asked for username and password next).\n".
         "If you dont have permission to access the database from this host\n".
         "you may want to install this ODL script on another machine or grant\n".
         "permissions before you continue. If the database does not exist, you\n".
         "probably should create it before continuing.\n";
   do {
      my @dbparts = ('DBI', 'mysql', 'dbreview');
      if ($config->{'database'}->[0] =~ /([^:]+):([^:]+):(.*)/)
      {
         @dbparts = ($1, $2, $3);
      }
   
      my $found = 0;
      do {
         print "\nDrivers: $driver_list\n";
         InputLine (\$dbparts[1], "Choose database driver from above list");
         foreach my $testdbidriver (@dbi_drivers)
         {
            if ($testdbidriver eq $dbparts[1])
            { $found = 1; }
         }
      } while (InvalidInput (($found == 0), "Invalid choice. Please choose again."));
      
      InputLine (\$dbparts[2], "Enter name of database");

      $dbparts[0] = 'DBI';
      $config->{'database'}->[0] = join (':', @dbparts);

      InputLine (\$config->{'dbusername'}->[0], "Enter the username to be used to connect");
      InputLine (\$config->{'dbpassword'}->[0], "Enter the password to be used to connect");

      print "Checking ... ";
      my $dbh = DBI->connect ($config->{'database'}->[0], $config->{'dbusername'}->[0], 
                              $config->{'dbpassword'}->[0], { PrintError=>0 });
      if (! defined $dbh)
      { 
         print "Cannot connect to the specified database with user/pass. Try again.\n";
         $fastforward = 0;
      }
      else
      {
         print "ok\n";
         $dbh->disconnect;
         $ok = 1;
      }
   } while ($ok == 0);
   

   # Database Table
   print "\n[DATABASE TABLE]\n";
   print "Within the database, multiple tables are needed for the indices\n".
         "Enter a unique table prefix for this component\n";
   print "Examples: dbunion, $ARGV[0]\n\n";
   do {
      InputLine (\$config->{'table'}->[0], "Table prefix");
   } while (InvalidInput (($config->{'table'}->[0] eq ''),
                          "This field cannot be left blank. Please enter a value."));


   # Repository Name
   print "\n[REPOSITORY NAME]\n";
   print "For identification purposes, you need to specify the full name of\n".
         "the component\n".
         "Example: JCDL ODL Box Archive\n\n";
   do {
      InputLine (\$config->{'repositoryName'}->[0], "Enter your repository name");
   } while (InvalidInput (($config->{'repositoryName'}->[0] eq ''),
                          "This field cannot be left blank. Please enter a value."));
   

   # Admin Email
   print "\n[ADMINISTRATOR EMAIL]\n";
   print "When your union archive is used by others, any problems\n".
         "will be reported to an administator, whose email must be provided\n".
         "Examples: admin\@odlxreview.org, provost\@university.edu.\n\n";
   do {
      InputLine (\$config->{'adminEmail'}->[0], "Enter your administrator's email");
   } while (InvalidInput (($config->{'adminEmail'}->[0] !~ /^[^@]+@[^@\.]+\.(.+)/), 
                          "That doesnt seem to be in the right format. Please try again."));


   # Record Limit
   print "\n[RECORD LIMIT]\n";
   print "Indicate the number of records to be issued in each batch\n".
         "before a resumptionToken is issued\n\n";
   do {
      InputLine (\$config->{'recordlimit'}->[0], "Record limit");
   } while (InvalidInput ((($config->{'recordlimit'}->[0] ne '') && ($config->{'recordlimit'}->[0] < 1)),
                          "This field must be a number and >0. Please try again."));


   # Archive Id
   print "\n[ARCHIVE ID]\n";
   print "What is the archive identifier of this archive ?\n".
         "Examples: odlxreview, $ARGV[0]\n\n";
   do {
      InputLine (\$config->{'archiveId'}->[0], "Archive ID");
   } while (InvalidInput (($config->{'archiveId'}->[0] eq ''),
                          "This field cannot be left blank. Please enter a value."));


   # Allow Read
   my $hostname = `hostname`;
   $hostname =~ s/[\r\n\t\s]//go;
   print "\n[ALLOW READ]\n";
   print "Enter the names (or IP addresses) of those machines that may use the\n".
         "OAI/ODL interfaces to read from the archive. Enter an empty line to end and\n".
         "remember you can use &delete to delete an entry.\n".
         "Examples: 127.0.0.1, $hostname\n\n";
   my $entry = 0;
   do {
      if ($#{$config->{'allowread'}} < $entry)
      {
         $config->{'allowread'}->[$entry] = '';
      }
      do {
         InputLine (\$config->{'allowread'}->[$entry], "Allow Read");
      } while (InvalidInput ((($config->{'allowread'}->[$entry] !~ /[^\.]+(\.[^\.]+)*/) && 
                              ($config->{'allowread'}->[$entry] ne '')),
                             "The machine name must be in dot-notation. Please try again."));
      $entry++;
   } while ($config->{'allowread'}->[$entry-1] ne '');


   # Allow Read
   print "\n[ALLOW WRITE]\n";
   print "Enter the names (or IP addresses) of those machines that may use the\n".
         "OAI/ODL interfaces to write to the archive. Enter an empty line to end and\n".
         "remember you can use &delete to delete an entry.\n".
         "Examples: 127.0.0.1, $hostname\n\n";
   $entry = 0;
   do {
      if ($#{$config->{'allowwrite'}} < $entry)
      {
         $config->{'allowwrite'}->[$entry] = '';
      }
      do {
         InputLine (\$config->{'allowwrite'}->[$entry], "Allow Write");
      } while (InvalidInput ((($config->{'allowwrite'}->[$entry] !~ /[^\.]+(\.[^\.]+)*/) && 
                              ($config->{'allowwrite'}->[$entry] ne '')),
                             "The machine name must be in dot-notation. Please try again."));
      $entry++;
   } while ($config->{'allowwrite'}->[$entry-1] ne '');


   WriteConfig ($config, $ARGV[0]);
   
   print "\n\n\nFinis.\n\n";
   
   print "Now you are ready to use the xreview engine\n".
         "Use $ARGV[0]/review.pl as the last bit of the baseURL for the archive interface\n\n";
}

main;
