#  ---------------------------------------------------------------------
#   XML implementation of ODL-Review (v1.0) - data provider
#    v1.0
#    March 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu
#     Digital Library Research Laboratory   |       www.dlib.vt.edu
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu
#  -------------------------------------------------------+-------------


package ODL::DBReview::DBReviewDP;

use Data::Dumper;
use POSIX;

use Pure::EZXML;
use ODL::DBReview::DBBuilder;

use ODL::Box::BoxDP;
use vars ('@ISA');
@ISA = ("ODL::Box::BoxDP");


# constructor
sub new
{
   my ($classname, $configfile) = @_;
   my $self = $classname->SUPER::new ($configfile);
   
   $self->{'odlsearch_xmlnsprefix'} = 'http://oai.dlib.vt.edu/ODL/Search/1.0';
   
   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->SUPER::dispose ();
}


# component-specific identification
sub Archive_Identify
{
   my ($self) = @_;

   my $id = $self->SUPER::Archive_Identify ();

   if (! exists $id->{'description'})
   {
      $id->{'description'} = [];
   }
   my $desc = {
      'odl-description' => [[
         {
            'xmlns' => $self->{xoai_xmlnsprefix}.'/odl-description',
            'xsi:schemaLocation' =>
                       $self->{xoai_xmlnsprefix}.'/odl-description '.
                       $self->{xoai_xmlnsprefix}.'/odl-description.xsd'
         },
         {
            'protocol' => 'ODL-Review',
            'version'  => '1.0',
            'mdorder'  => [ qw ( protocol version ) ]
         }
      ]]
   };
   push (@{$id->{'description'}}, $desc);

   $id;
}


# retrieve records from the source archive as required
sub Archive_FormatRecord
{
   my ($self, $hashref, $metadataFormat) = @_;

   if (defined $hashref->{$metadataFormat})
   {
      my $rowref = $hashref->{$metadataFormat};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           $rowref->{'metadata'},
                           $rowref->{'about'},
                           $rowref->{'status'}
                          );
   }
   elsif ((keys %$hashref) == 0)
   {
      return '';
   }
   else
   {
      my $firstkey = (keys %$hashref)[0];
      my $rowref = $hashref->{$firstkey};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           '',
                           '',
                           $rowref->{'status'}
                          );
   }
}


# get full list of mdps or list for specific identifier
sub Archive_ListMetadataFormats
{
   my ($self, $identifier) = @_;

   my ($metadataNamespace, $metadataSchema) = $self->SUPER::Archive_ListMetadataFormats ($identifier);

   ( $metadataNamespace, $metadataSchema );
}


# pass request through to source
sub Archive_GetRecord
{
   my ($self, $identifier, $metadataPrefix) = @_;

   my %records = ();

   if ($metadataPrefix eq 'odl_review')
   {
      my $sth = $self->{'dbh'}->prepare ("select identifier,status,datestamp,metadataPrefix,metadata,about from $self->{'table'} where identifier=\'$identifier\'");
      $sth->execute;

      my $rowref;
      while ($rowref = $sth->fetchrow_hashref)
      {
         $records{$rowref->{'metadataPrefix'}} = {
            identifier => $rowref->{'identifier'},
            datestamp  => $rowref->{'datestamp'},
            status     => $rowref->{'status'},
            metadata   => $rowref->{'metadata'},
            about      => $rowref->{'about'}
         };
      }

      $sth->finish;
   }
   elsif ($metadataPrefix eq 'resource_summary')
   {
      my $db = new ODL::DBReview::DBBuilder ($self->{'database'}, $self->{'dbusername'}, 
                                             $self->{'dbpassword'}, $self->{'table'});
      $records{'resource_summary'} = {
         identifier => $identifier,
         datestamp  => time,
         metadata   => $db->getResourceSummary ($identifier),
      };
      $db->dispose;
   }

   \%records;
}


# return list of records/identifiers
sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   if (($metadataPrefix eq 'editing_summary') || 
       ($metadataPrefix eq 'reviewing_summary') ||
       ($metadataPrefix eq 'children_summary') ||
       ($metadataPrefix eq 'open_summary') ||
       ($metadataPrefix eq 'notclosed_summary') ||
       ($metadataPrefix eq 'submitted_summary'))
   {
      my $db = new ODL::DBReview::DBBuilder ($self->{'database'}, $self->{'dbusername'},
                                             $self->{'dbpassword'}, $self->{'table'});
      my @recordlist;
      
      my $start = undef;
      my $stop = undef;
      if ($set =~ /([0-9]+)\/([0-9]+)\/(.*)/)
      {
         $start = $1;
         $stop = $2;
         $set = $3;
      }
      if ($set =~ /([0-9]+)\/([0-9]+)/)
      {
         $start = $1;
         $stop = $2;
         $set = '';
      }

      if ($metadataPrefix eq 'editing_summary')
      { @recordlist = $db->getEditingSummary ($set, $start, $stop); }
      if ($metadataPrefix eq 'reviewing_summary')
      { @recordlist = $db->getReviewingSummary ($set, $start, $stop); }
      elsif ($metadataPrefix eq 'children_summary')
      { @recordlist = $db->getChildren ($set, $start, $stop); }
      elsif ($metadataPrefix eq 'open_summary')
      { @recordlist = $db->getOpen ($start, $stop); }
      elsif ($metadataPrefix eq 'notclosed_summary')
      { @recordlist = $db->getNotClosed ($start, $stop); }
      elsif ($metadataPrefix eq 'submitted_summary')
      { @recordlist = $db->getSubmitted ($set, $start, $stop); }

      $db->dispose;
      
      my $count = pop @recordlist;
      
      my @recordlist2 = map { { $metadataPrefix => {
         identifier => $_->[0],
         datestamp  => time,
         metadata   => $_->[1],
         status     => '',
         about      => '',
      } } } @recordlist;
      
      my $responseContainer =
         "<hits ".
         "xmlns=\"$self->{'odlsearch_xmlnsprefix'}/responsecontainer\" ".
         "xsi:schemaLocation=\"$self->{'odlsearch_xmlnsprefix'}/responsecontainer ".
         "$self->{'odlsearch_xmlnsprefix'}/responsecontainer.xsd\">".
         $count.
         "</hits>";
      
      return ( \@recordlist2, '', $responseContainer, $metadataPrefix );
   }
   
   if ($set ne '')
   {
      # check for editor status for the resource
      if ($set =~ /^([^\/]+)\/(.*)$/)
      {
         $set = $1;

         my $tree = new ODL::DBReview::DBBuilder ($self->{'database'}, $self->{'dbusername'},
                                                  $self->{'dbpassword'}, $self->{'table'});
         if ($tree->isEditor ($2, $set) == 0)
         {
            return ([], '', '');
         }
         
         return $self->SUPER::Archive_ListRecords ($set, $from, $until, $metadataPrefix, $resumptionToken);
      }
   }
   
   ([], '', '');
}


# insert a new record into the archive
sub Archive_PutRecord
{
   my ($self, $identifier, $datestamp, $metadata, $metadataPrefix,
       $metadataNamespace, $metadataSchema, $about, $status, @sets) = @_;

   my $sth;

   # get unique identifier if necessary
   if ($identifier eq '')
   {
      $sth = $self->{'dbh'}->prepare ("lock tables $self->{'table'}admin write");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("select value from $self->{'table'}admin where variable=\'counter\'");
      $sth->execute;
      if (my $rowref = $sth->fetchrow_hashref)
      {
         $identifier = "oai:$self->{'archiveId'}:$rowref->{'value'}";
      }
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("update $self->{'table'}admin set value=value+1 where variable=\'counter\'");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("unlock tables");
      $sth->execute;
      $sth->finish;
   }

   if ($status eq 'deleted')
   {
      # delete everything
      $sth = $self->{'dbh'}->prepare ("update $self->{'table'} set status=\'deleted\' where identifier=\'$identifier\'");
      $sth->execute;
      $sth->finish;
   }
   else
   {
      # add basic metadata
      $sth = $self->{'dbh'}->prepare ("delete from $self->{'table'} where identifier=\'$identifier\' and metadataPrefix=\'$metadataPrefix\'");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("insert into $self->{'table'} set identifier=\'$identifier\', ".
                            "datestamp=\'$datestamp\', ".
                            "metadata=".$self->{'dbh'}->quote ($metadata).', '.
                            "metadataPrefix=\'$metadataPrefix\', ".
                            "about=".$self->{'dbh'}->quote ($about).', '.
                            "status=\'$status\'");
      $sth->execute;
      $sth->finish;

      # process transaction
      my $tree = new ODL::DBReview::DBBuilder ($self->{'database'}, $self->{'dbusername'},
                                               $self->{'dbpassword'}, $self->{'table'});
      my $setstring = $tree->add ($metadata, $identifier);
      $tree->dispose;

      # add sets for resources based on transaction result
      my @newsets = split (',', $setstring);
      $sth = $self->{'dbh'}->prepare ("delete from $self->{'table'}set where identifier=\'$identifier\'");
      $sth->execute;
      $sth->finish;
      foreach my $aset (@newsets)
      {
         $sth = $self->{'dbh'}->prepare ("insert into $self->{'table'}set set identifier=\'$identifier\', oaiset=\'$aset\'");
         $sth->execute;
         $sth->finish;
      }

      # add metadata namespace/schema
      $sth = $self->{'dbh'}->prepare ("delete from $self->{'table'}md where metadataPrefix=\'$metadataPrefix\' and metadataNamespace=\'$metadataNamespace\' and metadataSchema=\'$metadataSchema\'");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("insert into $self->{'table'}md set metadataPrefix=\'$metadataPrefix\', metadataNamespace=\'$metadataNamespace\', metadataSchema=\'$metadataSchema\'");
      $sth->execute;
      $sth->finish;
   }

   $identifier;
}


1;

